/**
* @brief HttpClient.h
* This file declares HttpClient class For Using WinInet
*
* @author	: nhn
* @date		: 2008-08-04
*/
#pragma once

#include <windows.h>
#include <tchar.h>
#include <wininet.h>
#include <vector>

/**
* @namespace tip_base
* This is base namespace in NUU Project
*
* @author	: nhn
* @date		: 2008-08-04
*/
namespace httpbase
{
	#define SAFE_DELETES(pObjs)		{ if( NULL != pObjs ){ delete [] pObjs; pObjs = NULL; } }
	#define SAFE_TASKMEMDELETES(pObj)		{ if( NULL != pObj ){ ::CoTaskMemFree(pObj); pObj = NULL; } }

	class IWriteBuffer;
	class IFileDownloadCallBack
	{
	public:
		virtual int Readed( DWORDLONG _current, DWORDLONG _total ) = 0;
	};

	// PRE-DEFINED CONSTANTS
	#define DEFAULT_AGENT_NAME			_T("NPhoto-Client")
	#define HTTP_VERB_GET				_T("GET")
	#define HTTP_VERB_POST				_T("POST")
	#define HTTP_ACCEPTYPE				_T("*/*")
	#define HTTP_ACCEPT					_T("Accept: */*\r\n")
	#define HTTP_CC_HOST				_T("Host: cc.naver.com")
	#define	HTTP_REFER_NAME				_T("REFERER: client://navertools")	

	const int SIZE_HTTP_BUFFER				= 100000;
	const int SIZE_HTTP_RESPONSE_BUFFER		= 100000;
	const int SIZE_HTTP_HEAD_LINE			= 2048;
	const int SIZE_HTTP_ARGUMENT_NAME		= 256;
	const int SIZE_HTTP_ARGUMENT_VALUE		= 1024*16;

	const int SIZE_HEADER_BUFFER			= 1024;
	const int SIZE_BUFFER					= 1024*8;
	const int SIZE_SMALL_BUFFER				= 256;

	/**
	* @class CNaver
	* This class supports Http Protocol For Using WinInet
	*
	* @author	: nhn
	* @date		: 2008-08-04
	*/
	class GenericHTTPClient
	{
	public:
		typedef struct GENERIC_HTTP_ARGUMENT
		{
			TCHAR	name[SIZE_HTTP_ARGUMENT_NAME];
			TCHAR	value[SIZE_HTTP_ARGUMENT_VALUE];
			DWORD	type;

			bool	operator==(const GENERIC_HTTP_ARGUMENT &argV)
			{
				return !lstrcmp(name, argV.name) && !lstrcmp(value, argV.value);
			}
		}GenericHTTPArgument;

		enum RequestMethod
		{
			RequestUnknown=0,
			RequestGetMethod=1,
			RequestPostMethod=2,
			RequestPostMethodMultiPartsFormData=3
		};

		enum TypePostArgument
		{													
			TypeUnknown=0,
			TypeNormal=1,
			TypeBinary=2
		};

		GenericHTTPClient();
		virtual ~GenericHTTPClient();

		static GenericHTTPClient::RequestMethod		GetMethod(int method);
		static GenericHTTPClient::TypePostArgument	GetPostArgumentType(int type);

		void DebugMessage();
		BOOL Connect( LPCTSTR cookies, LPCTSTR address, LPCTSTR agent = DEFAULT_AGENT_NAME,
			WORD port = INTERNET_DEFAULT_HTTP_PORT, LPCTSTR id = NULL, LPCTSTR password = NULL, int timeout = 3000 );
		BOOL Close();
		void InitializePostArguments();

		void AddPostArguments( LPCTSTR name, DWORD value );
		void AddPostArguments( LPCTSTR name, LPCTSTR value, BOOL is_binary = FALSE );

		BOOL Request( LPCTSTR cookies, LPCTSTR url, int method = GenericHTTPClient::RequestGetMethod,
						LPCTSTR filename = NULL, LPCTSTR agent = DEFAULT_AGENT_NAME, IFileDownloadCallBack *callback = NULL );
		BOOL RequestOfURI( LPCTSTR uri, int method = GenericHTTPClient::RequestGetMethod );
		DWORD GetReponseSize();
		void ParseHttpURL(LPCTSTR url, LPTSTR protocol, LPTSTR address, DWORD& port, LPTSTR uri);
		BOOL Response(LPCTSTR filename, DWORD &result_size, BYTE **requestBody, TCHAR **requestHeader );

		DWORD GetContentLength();
		BYTE* QueryHTTPResponse();
		DWORD	QueryHTTPResponseSize();
		LPCTSTR QueryHTTPResponseHeader();
		DWORD GetHttpResponseCode()
		{
			return m_dwHttpResponseCode;
		}

		bool m_bIsNclicks;
		void SetNclicks(bool bSet = true)
		{
			m_bIsNclicks = bSet;
		}

		// General Handler
		DWORD GetLastError();

		void GetContentType(LPCTSTR name, LPTSTR type);
	protected:
		BOOL GetHttpQueryHeaders(TCHAR** requestHeader);
		BOOL CheckReceiveSize( int filesize, int recvsize );
		DWORD ResponseOfBytes(PBYTE buffer, DWORD size, DWORD currentSize, DWORD filesize);
		BOOL RequestPost(LPCTSTR uri);
		BOOL RequestPostMultiPartsFormData(LPCTSTR uri);
		BOOL RequestGet(LPCTSTR uri);
		DWORD GetPostArguments( BYTE* argumetns, DWORD length );
		DWORD AllocMultiPartsFormData(PBYTE &in_buffer, LPCTSTR boundary = _T("--MULTI-PARTS-FORM-DATA-BOUNDARY-"));
		void FreeMultiPartsFormData(PBYTE &buffer);
		DWORD GetMultiPartsFormDataLength();
		BOOL SaveToBuffer( DWORD &filesize, DWORD content_length, IWriteBuffer* writeBuffer );
		IWriteBuffer* MakeWriteBuffer( LPCTSTR filename, DWORD content_length, BYTE **requestBody );

	protected:
		std::vector<GenericHTTPArgument> m_arguments;
		HINTERNET		m_http_open;
		HINTERNET		m_http_connection;
		HINTERNET		m_http_request;

		DWORD			m_content_length;
		DWORD			m_error;
		DWORD			m_result_size;

		// 200 or 404  HTTP ڵ, ʱⰪ0
		DWORD			m_dwHttpResponseCode;

		TCHAR			*m_http_response_header;
		BYTE			*m_http_response_html;
		IFileDownloadCallBack*	m_callback;
	};

};